# July 20, 2012

library(cobs)
library(ks)
library(logcondens)

# create samples for Figure 4
xx		<-	seq(-5,5,1)
B		<-	100000
CC1		<-	rep(0,B)		#convex
CC2		<-	rep(0,B)		#concave

pmt	<-	proc.time()
for(i in 1:B){
	error	<-	rnorm(length(xx))
	CC1[i]	<-	conreg(x=xx, y=xx^2+error, convex=TRUE, verbose=FALSE)$yf[6]
	CC2[i]	<-	conreg(x=xx, y=-xx^2+error, convex=FALSE, verbose=FALSE)$yf[6]
	print(i)	
}
proc.time()-pmt

write.table(CC1, "testCC1.out")
write.table(CC2, "testCC2.out")

# read in data (using already available results from above simulation)
CC1 <- read.table("testCC1.out")
CC1 <- CC1$x
CC2 <- read.table("testCC2.out")
CC2 <- CC2$x



zz		<-	seq(-4,4,length.out=1000)
f1		<-	kde(CC1, eval.points=zz, h=hpi(CC1))$estimate
f2		<-	kde(CC2, eval.points=zz, h=hpi(CC2))$estimate
mle1	<-	logConDens(CC1)
mle2	<-	logConDens(CC2)
f1.lcd	<-	evaluateLogConDens(zz, mle1, which=c(2,4))
f2.lcd	<-	evaluateLogConDens(zz, mle2, which=c(2,4))

# Figure 4 (top)
postscript("symmetry.eps", height=4, width=8, pointsize=16)
pdf("symmetry.pdf", height=4, width=8, pointsize=16)

par(las = 1, oma = c(0, 0, 0, 0), cex = 0.9, mar = c(2.5, 3, 1, 1))
plot(0, 0, xaxt = "n", type = "n", xlim = c(-3,3), xlab = "", ylim = c(0, 0.5), ylab = "pdf", main = "")
axis(1, at = seq(-5, 5, 1), labels = seq(-5, 5, 1))
lines(zz, f1, col="blue", lwd=1.5)
lines(zz, f2, col="red", lwd=1.5)
abline(v=0, lty=2)

dev.off()

# Figure 4 (bottom)
postscript("symmetry_log.eps", height=4, width=8, pointsize=16)
pdf("symmetry_log.pdf", height=4, width=8, pointsize=16)

par(las = 1, oma = c(0, 0, 0, 0), cex = 0.9, mar = c(2.5, 3, 1, 1))
plot(0, 0, xaxt = "n", type = "n", xlim = c(-4,4), xlab = "", ylim = c(-12, 0), ylab = "pdf", main = "")
axis(1, at = seq(-5, 5, 1), labels = seq(-5, 5, 1))
lines(zz, log(f1), col="blue", lwd=1.5)
lines(zz, log(f2), col="red", lwd=1.5)
lines(zz, log(f1.lcd[,5]), col="blue", lty=2)
lines(zz, log(f2.lcd[,5]), col="red", lty=2)
abline(v=0, lty=2)
rug(c(-2.30,1.90), col="blue", lwd=2)
rug(c(-1.92,2.28), col="red", lwd=2)

dev.off()

# Figure 3
postscript("symmetry_eg.eps", height=5, width=8, pointsize=16)
pdf("symmetry_eg.pdf", height=5, width=8, pointsize=16)

B		<-	25
xx		<-	seq(-5,5,1)
res		<-	matrix(rep(0,B*length(xx)), nrow=B)

set.seed(34757)
for(i in 1:B){
	error	<-	rnorm(length(xx))
	res[i,]	<-	conreg(x=xx, y=xx^2+error, convex=TRUE, verbose=FALSE)$yf
}

par(las = 1, oma = c(0, 0, 0, 0), cex = 0.9, mar = c(2.5, 3, 1, 1))
plot(0, 0, xaxt = "n", type = "n", xlim = c(-5,5), xlab = "", ylim = c(-3, 28), ylab = "pdf", main = "")
axis(1, at = seq(-5, 5, 1), labels = seq(-5, 5, 1))
for(i in 1:1000)  points(xx, xx^2+rnorm(length(xx)), col=gray(0.9))
for(i in 1:B)  lines(xx, res[i,], col="blue")
lines(xx, xx^2, lwd=2)
abline(h=0, lty=2)

dev.off()


