# 	September 15, 2012
#	empirical coverage sample simulation

source("functions.R")


#--------------------------------------
#--------------------------------------
#  normal distribution

xx0		<-	seq(-3,3, length.out=500)
f0		<-	dnorm(xx0)
c2		<-	(f0^3/24)^0.2
B		<-	1000
n		<-	1000

set.seed(8543048)
seed_vec	<-	round(10000000*runif(B),0)


#alpha		<-	0.05
zup			<-	2.7536	
zlo			<-	-2.4157

filename	<-	paste("cov_norm_n", n, ".out", sep="")
zzz			<-	file(filename, "w")

ptm			<-	proc.time()
for(i in 1:B){
	set.seed(seed_vec[i])
	data		<-	rnorm(n)
	ci			<-	calcCI(data, xx0, type=c("DR", "ks", "nrd"), htype=c("hns"))
	
	up_or		<-	ci$fhat-zlo*(n^(-0.4))*c2
	lo_or		<-	pmax(0,ci$fhat-zup*(n^(-0.4))*c2)
	
    cov_oracle  <-	(lo_or<=f0)*(f0<=up_or)
    cov_DR  	<-	(ci$lo_DR<=f0)*(f0<=ci$up_DR)
    cov_ks  	<-	(ci$lo_ks_hns<=f0)*(f0<=ci$up_ks_hns)
    cov_nrd  	<-	(ci$lo_nrd<=f0)*(f0<=ci$up_nrd)

	cat("oracle" , cov_oracle, "\n", file=zzz, sep=" ")
	cat("DR" , cov_DR, "\n", file=zzz, sep=" ")
	cat("ks" , cov_ks, "\n", file=zzz, sep=" ")
	cat("nrd" , cov_nrd, "\n", file=zzz, sep=" ")
	
	print(i)
		
}

close(zzz)
proc.time()-ptm

#----------------------------
# normal example picture (bootstrap is not included)

xx0			<-	seq(-3,3,length.out=500)

res1		<-	read.table("cov_norm_n1000.out")

emp_oracle	<-	as.matrix(res1[res1$V1=="oracle",2:501])
emp_DR		<-	as.matrix(res1[res1$V1=="DR",2:501])
emp_ks		<-	as.matrix(res1[res1$V1=="ks",2:501])
emp_nrd		<-	as.matrix(res1[res1$V1=="nrd",2:501])

cov_oracle	<-	apply(emp_oracle, 2, mean)
cov_DR		<-	apply(emp_DR, 2, mean)
cov_ks		<-	apply(emp_ks, 2, mean)
cov_nrd		<-	apply(emp_nrd, 2, mean)

postscript("cov_norm_n1000.eps", horizontal=FALSE, onefile=FALSE, height=3, width=9, pointsize=14)
pdf("cov_norm_n1000.pdf", onefile=FALSE, height=3, width=9, pointsize=14)

	par(pty="m", mar=c(2,2,0.5,2))
	plot(xx0, cov_oracle, type="l", ylim=c(0.90,1), xlab="", ylab="coverage", yaxt="no")
	axis(2, at=c(0.92, 0.95, 0.98), labels=c(0.92, 0.95, 0.98))
	lines(xx0, cov_DR, col=grey(0.8))
	lines(xx0, cov_ks, col="red")
	lines(xx0, cov_nrd, col="blue")
	lines(xx0, rep(0.95+2*sqrt(0.05*0.95/1000), 500), lty=2)
	lines(xx0, rep(0.95-2*sqrt(0.05*0.95/1000), 500), lty=2)
	rug(rnorm(1000), lwd=2)

dev.off()

