# Section 5.1 - simulated data example
library(logcondens)
library(ks)
library(evd)
source("functions.R")

# generate a standard gumbel data set
set.seed(7819217)
n	<-	500
x	<-	rgumbel(n)

# find log-concave MLE and plot the result
mle   	<- logConDens(x, smoothed=FALSE)
summary(mle)
plot(mle, which="density")

# find kernel-density estimator and plot the result
kd   	<- kde(x, h=hns(x))
plot(kd)
rug(x)

# MLE calculate 95% confidence interval for true density and add it to the plot
grid	<-	seq(min(x), max(x), length.out=200)
ci		<-	calcCI(x, grid, type="nrd")	

#postscript("gumbel_mle.eps", horizontal=FALSE, height=6, width=8, pointsize=15)
#pdf("gumbel_mle.pdf", onefile=FALSE, height=6, width=8, pointsize=15)

par(las=1, mar=c(2.5,3.5,0.5,0.5))
hist(x, n=25, col=gray(0.9), main="", freq=FALSE, xlab="", ylab="", ylim=c(0, 0.5), border=gray(0.5))
lines(grid, dgumbel(grid), lwd=2)
lines(grid, ci$fhat, col="red", lwd=2)
lines(grid, ci$lo_nrd, col="red", lwd=2, lty=2)
lines(grid, ci$up_nrd, col="red", lwd=2, lty=2)

#dev.off()

# KDE calculate 95% confidence interval for true density and add it to the plot
grid	<-	seq(min(x), max(x), length.out=200)
h   	<-	hns(x)
fhat	<-	kde(x, h=h, eval.points=grid)$estimate
fhat2	<-	kdde(x, h=h, deriv.order=2, eval.points=grid)$estimate
mu2 	<-  1
k2  	<-  1/sqrt(4*pi)
l   	<-  fhat - (h^2/2)*fhat2*mu2 - 1.96*sqrt(fhat*k2/(n*h))
u   	<-  fhat - (h^2/2)*fhat2*mu2 + 1.96*sqrt(fhat*k2/(n*h))

postscript("gumbel_kde.eps", horizontal=FALSE, height=6, width=8, pointsize=15)
pdf("gumbel_kde.pdf", onefile=FALSE, height=6, width=8, pointsize=15)

par(las=1, mar=c(2.5,3.5,0.5,0.5))
hist(x, n=25, col=gray(0.9), main="", freq=FALSE, xlab="", ylab="", ylim=c(0, 0.5), border=gray(0.5))
lines(grid, dgumbel(grid), lwd=2)
lines(grid, fhat, col="blue", lwd=2)
lines(grid, u, col="blue", lwd=2, lty=2)
lines(grid, l, col="blue", lwd=2, lty=2)

dev.off()

# Section 5.2 - real-world data example

# call required functions
library(logcondens)
library(ks)
source("functions.R")

# prepare data set
data(reliability)
x.rel 	<- sort(reliability)
n     	<- length(x.rel)

# find log-concave MLE and plot the result
mle   	<- logConDens(x.rel, smoothed=FALSE)
summary(mle)
plot(mle, which="density")

# calculate 95% confidence interval for true density and add it to the plot
grid	<-	seq(min(x.rel), max(x.rel), length.out=200)
ci		<-	calcCI(x.rel, grid, type="nrd")
ci2		<-	calcCIboot(x.rel, grid)	# N.B.  this takes a few minutes to complete!


postscript("reliable.eps", horizontal=FALSE, height=6, width=8, pointsize=15)
pdf("reliable.pdf", onefile=FALSE, height=6, width=8, pointsize=15)

par(las=1, mar=c(2.5,3.5,0.5,0.5))
hist(x.rel, n=25, col=gray(0.9), main="", freq=FALSE, xlab="", ylab="", ylim=c(0, 0.0065), border=gray(0.5))
lines(grid, ci$fhat, col="red", lwd=2)
lines(grid, ci$lo_nrd, col="red", lwd=2, lty=5)
lines(grid, ci$up_nrd, col="red", lwd=2, lty=5)
lines(grid, ci2$lo, col="red", lwd=2, lty=3)
lines(grid, ci2$up, col="red", lwd=2, lty=3)

dev.off()

