# September 15, 2012

library(logcondens)
library(ks)
library(evd)

f0tilde    <-	function(loc, yy, fhat, dy, h){sum(dnorm(yy,loc,h)*fhat)*dy}
f1tilde		<-	function(loc, yy, fhat, dy, h){sum(-(loc-yy)*dnorm(yy,loc,h)*fhat)*dy/(h^2)}
f2tilde		<-	function(loc, yy, fhat, dy, h){sum((((loc-yy)/h)^2-1)*dnorm(yy,loc,h)*fhat)*dy/(h^2)}

c2hat	<-	function(data, xx0, type, htype="hns"){

				c2_DR		<-	numeric(0)
				c2_ks_hscv	<-	numeric(0)
				c2_ks_hlscv	<-	numeric(0)
				c2_ks_hpi	<-	numeric(0)
				c2_ks_hns	<-	numeric(0)
				c2_nrd		<-	numeric(0)
					
				spread		<-	diff(range(data))
				grid		<-	seq(min(data)-0.5*spread, max(data)+0.5*spread, length.out=2*1000)
				dy			<-	min(diff(grid))

				mle			<-	logConDens(data)
				fmle		<-	evaluateLogConDens(grid, mle, which=2)
				fhat		<-	fmle[,3]					
		
				if("DR" %in% type){
					h_DR		<-	sqrt(mle$sig^2-LocalVariance(x=mle$x,phi=mle$phi))
					f0			<-	function(loc) f0tilde(loc, grid, fhat, dy, h=h_DR)
					f0_DR		<-	unlist(lapply(xx0, f0))
					f1			<-	function(loc) f1tilde(loc, grid, fhat, dy, h=h_DR)
					f1_DR		<-	unlist(lapply(xx0, f1))
					f2			<-	function(loc) f2tilde(loc, grid, fhat, dy, h=h_DR)
					f2_DR		<-	unlist(lapply(xx0, f2))
					rm(f0,f1,f2)	
					c2_DR		<-	f0_DR*abs(f0_DR*f2_DR-(f1_DR)^2)
					c2_DR		<-	(c2_DR/24)^0.2
									
				}
					
				if("ks" %in% type){
					
					if("hscv" %in% htype){						
						h			<-	hscv(data)
						f0_ks		<-	kdde(x=data, eval.points=xx0, h=h, deriv.order=0)$estimate
						f1_ks		<-	kdde(x=data, eval.points=xx0, h=h, deriv.order=1)$estimate
						f2_ks		<-	kdde(x=data, eval.points=xx0, h=h, deriv.order=2)$estimate
						c2_ks		<-	f0_ks*abs(f0_ks*f2_ks-(f1_ks)^2)
						c2_ks_hscv	<-	(c2_ks/24)^0.2
						}

					if("hlscv" %in% htype){						
						h			<-	hlscv(data)
						f0_ks		<-	kdde(x=data, eval.points=xx0, h=h, deriv.order=0)$estimate
						f1_ks		<-	kdde(x=data, eval.points=xx0, h=h, deriv.order=1)$estimate
						f2_ks		<-	kdde(x=data, eval.points=xx0, h=h, deriv.order=2)$estimate
						c2_ks		<-	f0_ks*abs(f0_ks*f2_ks-(f1_ks)^2)
						c2_ks_hlscv	<-	(c2_ks/24)^0.2
						}

					if("hpi" %in% htype){						
						h			<-	hpi(data)
						f0_ks		<-	kdde(x=data, eval.points=xx0, h=h, deriv.order=0)$estimate
						f1_ks		<-	kdde(x=data, eval.points=xx0, h=h, deriv.order=1)$estimate
						f2_ks		<-	kdde(x=data, eval.points=xx0, h=h, deriv.order=2)$estimate
						c2_ks		<-	f0_ks*abs(f0_ks*f2_ks-(f1_ks)^2)
						c2_ks_hpi	<-	(c2_ks/24)^0.2
						}

					if("hns" %in% htype){						
						h			<-	hns(data)
						f0_ks		<-	kdde(x=data, eval.points=xx0, h=h, deriv.order=0)$estimate
						f1_ks		<-	kdde(x=data, eval.points=xx0, h=h, deriv.order=1)$estimate
						f2_ks		<-	kdde(x=data, eval.points=xx0, h=h, deriv.order=2)$estimate
						c2_ks		<-	f0_ks*abs(f0_ks*f2_ks-(f1_ks)^2)
						c2_ks_hns	<-	(c2_ks/24)^0.2
						}
					
				}
		
				if("nrd" %in% type){
					fmle		<-	evaluateLogConDens(xx0, mle, which=4)
					ftilde		<-	fmle[,5]
					c2_nrd		<-	ftilde^3/mle$sig^2					
					c2_nrd		<-	(c2_nrd/24)^0.2
									
				}
	
				return(list(DR=c2_DR, ks_hscv=c2_ks_hscv, ks_hlscv=c2_ks_hlscv, ks_hpi=c2_ks_hpi,ks_hns=c2_ks_hns, nrd=c2_nrd))
	
}

calcCI		<-	function(data, xx0, alpha=0.05, type="ks", htype="hns"){

				mle			<-	logConDens(data)
				fmle		<-	evaluateLogConDens(xx0, mle, which=2)
				fhat		<-	fmle[,3]
				n			<-	length(data)					
				c2fit		<-	c2hat(data=data, xx0=xx0, type=type, htype=htype)

				if(alpha==0.05){
				zup			<-	2.7536	
				zlo			<-	-2.4157
				}
	
				up_DR		<-	fhat-zlo*(n^(-0.4))*c2fit$DR
				up_ks_hscv	<-	fhat-zlo*(n^(-0.4))*c2fit$ks_hscv
				up_ks_hlscv	<-	fhat-zlo*(n^(-0.4))*c2fit$ks_hlscv
				up_ks_hpi	<-	fhat-zlo*(n^(-0.4))*c2fit$ks_hpi
				up_ks_hns	<-	fhat-zlo*(n^(-0.4))*c2fit$ks_hns
				up_nrd		<-	fhat-zlo*(n^(-0.4))*c2fit$nrd
	
				lo_DR		<-	pmax(0,fhat-zup*(n^(-0.4))*c2fit$DR)
				lo_ks_hscv	<-	pmax(0,fhat-zup*(n^(-0.4))*c2fit$ks_hscv)
				lo_ks_hlscv	<-	pmax(0,fhat-zup*(n^(-0.4))*c2fit$ks_hlscv)
				lo_ks_hpi	<-	pmax(0,fhat-zup*(n^(-0.4))*c2fit$ks_hpi)
				lo_ks_hns	<-	pmax(0,fhat-zup*(n^(-0.4))*c2fit$ks_hns)
				lo_nrd		<-	pmax(0,fhat-zup*(n^(-0.4))*c2fit$nrd)
	
	
				return(list(fhat=fhat, up_DR=up_DR, lo_DR=lo_DR, up_ks_hscv=up_ks_hscv, lo_ks_hscv=lo_ks_hscv, up_ks_hlscv=up_ks_hlscv, lo_ks_hlscv=lo_ks_hlscv, up_ks_hpi=up_ks_hpi, lo_ks_hpi=lo_ks_hpi, up_ks_hns=up_ks_hns, lo_ks_hns=lo_ks_hns, up_nrd=up_nrd, lo_nrd=lo_nrd))
		
}

# parametric percentile bootstrap
calcCIboot	<-	function(data, xx0, alpha=0.05, BB=500){

				mle			<-	logConDens(data)
				fmle		<-	evaluateLogConDens(xx0, mle, which=2)
				fhat		<-	fmle[,3]

				fhat_star	<-	matrix(rep(0,BB*length(grid)), nrow=BB)
				for(j in 1:BB){
							data_star		<-	sample(data, n, replace=TRUE)
							mle_star		<-	logConDens(data_star)
							fmle_star		<-	evaluateLogConDens(grid, mle_star, which=2)
							fhat_star[j,]	<-	fmle_star[,3]		
				}
	
				q_low		<-	apply(fhat_star, 2, quantile, probs=alpha/2)
				q_upp		<-	apply(fhat_star, 2, quantile, probs=1-alpha/2)

				return(list(fhat=fhat, up=q_upp, lo=q_low))
		
}
